(function() {

if (!self.eAu) { self.eAu = {}; }
if (eAu.Bookmarks) { return; }

var bookmarksOrder = [];
var bookmarksHash = {};
var DS;

function getId(node) {
	return _.isString(node) ? node : _Xml.id(node);
}

function getNode(node) {
	return _.isString(node) ? _Xml.$(head, node) : node;
}

function saveBookmarks() {
	DS.set('!b', bookmarksOrder.concat([]).reverse().join(','));
}

function initData(DataStorageEngine) {
	DS = DataStorageEngine;
	var bookmarksRaw = DS.get('!b') || '';
	
	var bookmarks = bookmarksRaw.split(',');
	if (bookmarks.length == 1 && !bookmarks[0]) {
		return;
	}

	for (var i = bookmarks.length - 1; i >= 0; --i) {
		var bId = bookmarks[i];
		var node = _Xml.$(head, bId);
		if (!node) {
			continue;
		}

		bookmarksOrder.push(bId)
		bookmarksHash[bId] = {
			title: _Xml.title(node),
			id: bId
		}
	}
}

function addBookmark(node) {
	if (!canAddBookmark(node)) { return; }

	var id = getId(node);

	if (bookmarksHash[id]) {
		// Достаточно пересортировать элементы
		var index = _.indexOf(bookmarksOrder, id);
		bookmarksOrder.splice(index, 1);
	} else {
		node = getNode(node);
		bookmarksHash[id] = {
			title: _Xml.title(node),
			id: id
		};
	}
	bookmarksOrder.unshift(id);

	saveBookmarks();
}

function canAddBookmark(node) {
	var id = getId(node);

	if (bookmarksHash[id]) {
		return true;
	}

	node = getNode(node);

	return !!node && DS.can('!b', bookmarksOrder.join(',') + ',' + id);
}

function deleteBookmark(node) {
	var id = getId(node);

	if (!bookmarksHash[id]) {
		return;
	}

	var index = _.indexOf(bookmarksOrder, id);
	bookmarksOrder.splice(index, 1);
	delete bookmarksHash[id];

	saveBookmarks();
}

function clearBookmarks() {
	bookmarksHash ={};
	bookmarksOrder = [];
	
	saveBookmarks();
}

function listBookmarks(limit) {
	var ret = [];

	limit = (limit || -1) < 0 ? bookmarksOrder.length : limit;

	limit = limit > bookmarksOrder.length ? bookmarksOrder.length : limit;

	for (var i = 0; i < limit; ++i) {
		ret.push(bookmarksHash[ bookmarksOrder[i] ]);
	}

	return ret;
}

function countBookmarks() {
	return bookmarksOrder.length;
}

eAu.Bookmarks = {
	init: initData,
	add: addBookmark,
	del: deleteBookmark,
	list: listBookmarks,
	can: canAddBookmark,
	clear: clearBookmarks,
	count: countBookmarks
};

})();