(function() {

if (!eAu) { self.eAu = {}; }
if (eAu.LMS) { return; }

// Shorthand for function
var U = _.isUndefined;

var Log = {
	log: function(str) {
		if (!Lms.debug) {
			return;
		}

		try {
			console.log(str);
		} catch (error) {}
	},
	error: function(str) {
		if (!Lms.debug) {
			return;
		}

		try {
			console.error(str);
		} catch (error) {}
	}
};

function prePad(num) {
	return ((num >= 0 && num <= 9) || (num >= 100 && num <= 999))
		? '0'+num.toString()
		: num.toString();
}

var Lms = {
	ApiTypes: ['scorm 2004', 'scorm 1.2'],
	ApiType: window.undefined,
	Api: window.undefined,
	Initialized: false,
	Terminated: false,
	Data: {},
	Time: {
		Start: 0,
		Finish: 0
	},
	GetTimeSpan: function() {
		var timeSpan = new Date(this.Time.Finish - this.Time.Start);
		
		if (this.ApiType == 'scorm 1.2') {
			var timestamp = timeSpan.getTime();
			var hrs = prePad(parseInt(timestamp / 1000 / 3600));
			timestamp -= hrs * 1000 * 3600;
			var min = prePad(parseInt(timestamp / 1000 / 60));
			timestamp -= min * 1000 * 60;
			var sec = prePad(parseInt(timestamp / 1000));
			timestamp -= sec * 1000;
			var msec = Math.floor(timestamp / 1000);

			return hrs +':'+ min +':'+ sec +'.'+ msec;
		} else {
			var time = {
				S:  Math.floor( (timeSpan.getUTCSeconds() + timeSpan.getUTCMilliseconds() / 1000) * 100 ) / 100,
				M:  timeSpan.getUTCMinutes(),
				H:  timeSpan.getUTCHours(),
				D:  timeSpan.getUTCDate() - 1,
				m:  timeSpan.getUTCMonth(),
				Y:  timeSpan.getUTCFullYear() - 1970
			};
			return _.reduce('PYMDTHmS'.split(''), function(memo, key) {
				return _.isNumber(time[key]) && !_.isNaN(time[key]) && time[key] >= 0.1
					? memo + String(time[key]) + key.toUpperCase()
					: memo + (_.isUndefined(time[key]) ? key.toUpperCase() : '');
			}, '');
		}
		return "";
	},
	FindApi: function() {
		var TheApi = null;
		var This = this;

		function FindApiInWindow(win) {
			var findApiTries = 0;

			var APIName = (This.ApiType == 'scorm 1.2')
				? 'API'
				: 'API_1484_11';

			while (!win[APIName] && !!win.parent && win.parent != win) {
				findApiTries++;
				if (findApiTries > 15) {
					return null;
				}
				win = win.parent;
			}

			return (!!win[APIName] ? win[APIName] : null);
		}

		try {
			TheApi = FindApiInWindow(window);

			if (!TheApi && !!window.opener) {
				TheApi = FindApiInWindow(window.opener);
			}

			if (!TheApi && !!top.opener) {
				TheApi = FindApiInWindow(top.opener);
			}
		} catch (error) {
			Log.error('FindApi exception: '+ error);
		}

		return TheApi;
	},
	Error: function() {
		return ((this.ApiType == 'scorm 1.2')
			? this.Api.LMSGetLastError()
			: this.Api.GetLastError()) != '0';
	},
	PrintError: function() {
		var code = (this.ApiType == 'scorm 1.2')
			? this.Api.LMSGetLastError()
			: this.Api.GetLastError();

		if (code == '0') {
			return;
		}

		var ErrorString = (this.ApiType == 'scorm 1.2')
			? this.Api.LMSGetErrorString(code.toString())
			: this.Api.GetErrorString(code.toString());

		var ErrorDiagnostic = (this.ApiType == 'scorm 1.2')
			? this.Api.LMSGetDiagnostic(code)
			: this.Api.GetDiagnostic(code);

		Log.error('ERROR:\n  code - '+ code +'\n  description - '+ ErrorString +'\n  diagnostic - '+ ErrorDiagnostic);
	},
	Get: function(name) {
		var value = ((this.ApiType == 'scorm 1.2')
			? this.Api.LMSGetValue(name)
			: this.Api.GetValue(name)) || '';
		var hasError = this.Error();

		Log.log('GET: '+ name + ' == '+ (hasError ? '(error getting value)' : value));
		if (hasError) {
			this.PrintError();
		}

		return value;
	},
	Set: function(name, varValue) {
		if (this.Data.mode != 'normal' && this.Data.mode != '') {
			Log.log('SET: SCO is launched without intention to modify anything');
			return false;
		}

		var value = (this.ApiType == 'scorm 1.2')
			? this.Api.LMSSetValue(name, varValue)
			: this.Api.SetValue(name, varValue);
		var hasError = value == 'false' && this.Error();

		Log.log('SET: '+ name +' = '+ varValue);
		if (hasError) {
			this.PrintError();
		}

		return value == 'true';
	},
	Commit: function() {
		var value = (this.ApiType == 'scorm 1.2')
			? this.Api.LMSCommit('')
			: this.Api.Commit('');

		var hasError = value == 'false' && this.Error();

		Log.log('COMMIT: '+ (hasError ? 'error' : 'success'));
		if (hasError) {
			this.PrintError();
		}

		return value == 'true';
	},
	Initialize: function() {
		var value = (this.ApiType == 'scorm 1.2')
			? this.Api.LMSInitialize('')
			: this.Api.Initialize('');

		var hasError = value == 'false' && this.Error();

		Log.log('INITIALIZE'+ (hasError ? 'error' : 'success'));
		if (hasError) {
			this.PrintError();
		}

		return value == 'true';
	},
	Init: function() {
		if (this.ApiType == 'scorm 1.2') {
			return this.Init12();
		} else {
			return this.Init2004();
		}
	},
	Init12: function() {
		if (!this.Initialize()) {
			return false;
		}

		this.Data.mode     = this.Get('cmi.core.lesson_mode');

		var lesson_status  = this.Get('cmi.core.lesson_status');
		this.Data.success_status    = 'unknown';
		this.Data.completion_status = 'not attempted';
		if (/incomplete|completed/.test(lesson_status)) {
			this.Data.completion_status = lesson_status;
		} else if (/passed|failed/.test(lesson_status)) {
			this.Data.success_status = lesson_status;
			this.Data.completion_status = 'completed';
		} else if (/unknown/.test(lesson_status)) {
			this.Set('cmi.core.lesson_status', 'not attempted');
		}

		this.Commit();

		this.Time.Start = new Date();
		this.Initialized = true;

		return true;
	},
	Init2004: function() {
		if (!this.Initialize()) {
			return false;
		}

		this.Data.mode     = this.Get('cmi.mode');

		this.Data.success_status = this.Get('cmi.success_status');

		if (this.Get('cmi.completion_status') == 'unknown') { 
			this.Set('cmi.completion_status', 'not attempted');
		}
		this.Data.completion_status = this.Get('cmi.completion_status');

		this.Commit();

		this.Time.Start = new Date();
		this.Initialized = true;

		return true;
	},
	Finish: function() {
		var value = (this.ApiType == 'scorm 1.2')
			? this.Api.LMSFinish('')
			: this.Api.Terminate('');
		var hasError = value == 'false' && this.Error();

		Log.log('FINISH: '+ (hasError ? 'error' : 'success'));
		if (hasError) {
			this.PrintError();
		}

		return value == 'true';
	}
};

(function preInit (instance) {
	for (var i = 0, length = Lms.ApiTypes.length; i < length; ++i) {
		Lms.ApiType = Lms.ApiTypes[i];
		Lms.Api = Lms.FindApi();
	}
})();

eAu.LMS = {
	debug: function(flag) {
		Lms.debug = !!flag;
		if (Lms.debug) {
			this._Lms = Lms;
		} else {
			this._Lms = null;
		}
	},
	initialize: function() {
		var value = false;

		if (Lms.Initialized) {
			return true;
		}

		if (!Lms.Api) {
			Lms.ApiType = window.undefined;
		} else {
			value = Lms.Init();
		}

		if (!Lms.Api && window.location.href.indexOf('http') == 0) {
			Log.error('eAu.LMS.initialize: can\'t find Api instance');
		}

		Lms.Initialized = value;
		Lms.Terminated = window.undefined;
		return value;
	},
	setLessonStatus: function(status) {
		if (!Lms.Initialized) {
			return;
		}

		var oldSuccessStatus = Lms.Data.success_status;
		var oldCompletionStatus = Lms.Data.completion_status;

		if ((/incomplete/.test(status) && !/completed/.test(Lms.Data.completion_status)) || /completed/.test(status)) {
			Lms.Data.completion_status = status;
		} else if (/passed|failed/.test(status)) {
			Lms.Data.success_status = status;
			if (/failed/.test(status)) {
				Lms.Data.completion_status = 'incomplete';
			} else {
				Lms.Data.completion_status = 'completed';
			}
		}

		if (Lms.ApiType == 'scorm 1.2') {
			Lms.Set(
				'cmi.core.lesson_status',
				Lms.Data.success_status == 'unknown'
					? Lms.Data.completion_status
					: Lms.Data.success_status
			);
		} else {
			if (oldSuccessStatus != Lms.Data.success_status) {
				Lms.Set('cmi.success_status',    Lms.Data.success_status);
			}
			if (oldCompletionStatus != Lms.Data.completion_status) {
				Lms.Set('cmi.completion_status', Lms.Data.completion_status);
			}
		}
	},
	setScoredLessonData: function(raw, min, max) {
		if (!Lms.Initialized) {
			return;
		}

		min = parseFloat(min) || 0.0;
		max = parseFloat(max) || 100.0;
		raw = parseFloat(raw) || 0.0;
		if (Lms.ApiType == 'scorm 1.2') {
			raw = (max - min == 0)
				? 0
				: Math.round((raw - min) * 100 / (max - min));

			Lms.Set('cmi.core.score.min', "0");
			Lms.Set('cmi.core.score.max', "100");
			Lms.Set('cmi.core.score.raw', String(raw));
		} else {
			Lms.Set('cmi.score.min', String(min));
			Lms.Set('cmi.score.max', String(max));
			Lms.Set('cmi.score.raw', String(raw));
		}
	},
	getSuspendData: function () {
		if (!Lms.Initialized) {
			return "";
		}
		return Lms.Get('cmi.suspend_data');
	},
	setSuspendData: function (data) {
		if (!Lms.Initialized || !_.isString(data)) {
			return;
		}
		Lms.Set('cmi.suspend_data', data);
		Lms.Commit();
	},
	navRequest: function(type, db_id) {
		if (!Lms.Initialized || Lms.ApiType == 'scorm 1.2') {
			return false;
		}
		if (db_id) {
			db_id = db_id.replace(/^{(.*)}$/, 'ITEM-ID-$1-eAu');
		}
		switch (type) {
		case 'choice':
			Lms.Set('adl.nav.request', '{target='+ db_id +'}choice');
			break;
		}
		return true;
	},
	terminate: function(location) {
		if (!Lms.Initialized || Lms.Terminated) {
			return;
		}
		Lms.Terminated = true;

		var exit = Lms.ApiType == 'scorm 1.2'
			? ''
			: 'normal';

		Lms.Time.Finish = new Date();
		var SessionTime = Lms.GetTimeSpan();

		if (Lms.Data.completion_status == 'not attempted') {
			this.setLessonStatus('completed');
		} else if (Lms.Data.completion_status == 'incomplete') {
			exit = 'suspend';
		}

		if (Lms.ApiType == 'scorm 1.2') {
			Lms.Set('cmi.core.session_time', SessionTime);
			Lms.Set('cmi.core.exit', exit);
		} else {
			Lms.Set('cmi.session_time', SessionTime);
			Lms.Set('cmi.exit', exit);
		}

		Lms.Commit();
		Lms.Finish();
		Lms.Initialized = window.undefined;
		return Lms.Terminated;
	},
	ok: function() {
		return !!Lms.Api;
	}
}

})();

$(window).bind('beforeunload unload', function() {
	eAu.LMS.terminate();
});
