;(function (root) {
"use strict";

var CurrentTest = null
  , DS
  , _savedScores = []
  , _emptyValue = {}
  , _cachedTests = {};
var NAVIGATION_MODE_FORWARD       = 0
  , NAVIGATION_MODE_BIDIRECTIONAL = 1
  , NAVIGATION_MODE_ALLOWSKIP     = 2;
var SELECTION_MODE_EQUAL          = 0
  , SELECTION_MODE_FIXED          = 1;
var questionNodes = ['question', 'question-exercise'];
var QRightAnswer
  , QScorers
  , QGetters
  , QuestionClassMethods
  , QuestionQueueClass
  , TestClass;
var norm = eAu.String.normalizeSpaces
  , trim = eAu.String.trim
  , attr = _.bind(_Xml.attr, _Xml)
  , pi = function (item) { return parseInt(item, 10); }
  , xid = _.bind(_Xml.id, _Xml)
  , str = _.bind(_.isString, _)
  , udef = _.bind(_.isUndefined, _);

if (!udef(root.eAu.Test)) { return; }

// Функция получает id ноды
// если была передана строка, то она и возвращается
// если был передан XML элемент, то возвращается его id
function getId (node) {
	return str(node) ? node : xid(node);
}

// Функция получает XML элемент по id
// если была передана строка, то возвращается элемент с указанным id
// если был передан XML элемент, то он и возвращается
function getNode (node, nodeName) {
	var ac = getNode,
	    id = getId(node);
	ac.cache = ac.cache || {};
	if (udef(ac.cache[id])) {
		ac.cache[id] = str(node) ? _Xml.$(head, node, nodeName) : node;
	}
	return ac.cache[id];
}

function isQuestionNode (idOrNodeName) {
	var qRegEx = new RegExp('^(' + questionNodes.join('|') + ')$');
	return qRegEx.test(String(
		getNode(idOrNodeName) ? _Xml.name(getNode(idOrNodeName)) : idOrNodeName
	));
}

function setCachedTest (test) {
	var id = xid(test.getTestNode());
	if (!udef(_cachedTests[id])) {
		return;
	}
	_cachedTests[id] = test;
}

function getCachedTest (nodeOrId) {
	var id = getId(nodeOrId);
	if (!udef(_cachedTests[id])) {
		return _cachedTests[id];
	}
}

// Собирает все xml ноды с вопросами в ноде <test>
function collectAllQuestions (test) {
	return _Xml.select(test, _.reduce(questionNodes, function (memo, item) {
		memo.push('descendant::' + item);
		return memo;
	}, []).join('|'));
}

// Возвращает все вопросы (их Xml ноды)
// для указанного теста (<test>)
function getQuestionsCache (id) {
	var ac = getQuestionsCache;
	ac.cache = ac.cache || {};

	if (!ac.cache[getId(id)]) {
		ac.cache[getId(id)] = collectAllQuestions(getNode(id, 'test'));
	}
	return ac.cache[getId(id)];
}

// Обработка атрибутов тега <test>
function parseTestOptions (test, questions) {
	var options = { };

	// Режим навигации по тесту
	// 0 - нельзя пропускать вопросы, нельзя возвращаться назад
	// 1 - с возможностью возврата к предыдущим вопросам
	// 2 - с возможностью пропускать вопросы
	options['navigationMode'] = pi(attr(test, 'navigation-mode'))
			&& pi(attr(test, 'navigation-mode')) > 0
			&& pi(attr(test, 'navigation-mode')) <= 2
		? pi(attr(test, 'navigation-mode'))
		: 0;
	attr(test, 'navigation-mode', String(options['navigationMode']));

	// Количество вопросов в тесте (0 - все вопросы)
	options['numberOfQuestions'] = pi(attr(test, 'number-of-questions'))
			&& pi(attr(test, 'number-of-questions')) > 0
		? pi(attr(test, 'number-of-questions'))
		: questions.length;
	attr(test, 'number-of-questions', String(options['numberOfQuestions']));

	// Ограничение времени на прохождение всего теста (0 - не ограничено)
	options['timeLimit'] = pi(attr(test, 'time-limit'))
			&& pi(attr(test, 'time-limit')) > 0
		? pi(attr(test, 'time-limit'))
		: 0;
	attr(test, 'time-limit', String(options['timeLimit']));

	// Количество попыток запуска теста (0 - не ограничено)
	// TODO: NotImplemented
	// TODO: RangeCheck
	options['numberOfTestTryes'] = pi(attr(test, 'number-of-test-tryes')) > 0
		? pi(attr(test, 'number-of-test-tryes'))
		: 0;
	attr(test, 'number-of-test-tryes', String(options['numberOfTestTryes']));

	// перемешивать вопросы
	options['shuffleQuestions'] = attr(test, 'shuffle-questions') == 'true'
		? true
		: false;
	attr(test, 'shuffle-questions', String(options['shuffleQuestions']));

	// Позволять досрочно завершать тестирование
	options['allowTestInterrupt'] = attr(test, 'allow-test-skip') == 'true'
		? true
		: false;
	attr(test, 'allow-test-skip', String(options['allowTestInterrupt']));

	// Показывать общие результаты тестирования
	options['showTestStats'] = attr(test, 'show-test-stats') == 'true'
		? true
		: false;
	attr(test, 'show-test-stats', String(options['showTestStats']));

	// Показывать общие результаты тестирования
	options['showRightAnswers'] = attr(test, 'show-right-answers') !== 'false'
		? true
		: false;
	attr(test, 'show-right-answers', String(options['showRightAnswers']));

	// Режим выборки вопросов для теста
	// 0 - одинаковое количество вопросов из каждой темы
	// 1 - количество вопросов задано явно
	options['selectionMode'] = pi(attr(test, 'selection-mode'))
			&& pi(attr(test, 'selection-mode')) > 0
			&& pi(attr(test, 'selection-mode')) <= 1
		? pi(attr(test, 'selection-mode'))
		: 0;
	attr(test, 'selection-mode', String(options['selectionMode']));

	// Процент баллов, который необходимо получить что-бы пройти тест
	options['passedPC'] = pi(attr(test, 'passed-pc'))
			&& pi(attr(test, 'passed-pc')) > 0
			&& pi(attr(test, 'passed-pc')) <= 100
		? pi(attr(test, 'passed-pc'))
		: 0;
	attr(test, 'passed-pc', String(options['passedPC']));
	
	// Количество вопросов на страницу
	options['itemsPerPage'] = pi(attr(test, 'ipp'))
			&& pi(attr(test, 'ipp')) >= 0
		? pi(attr(test, 'ipp'))
		: 1;
	attr(test, 'ipp', String(options['itemsPerPage']));

	return options;
}

// Возвращает обработанные настройки теста
function getOptionsCache (id) {
	var ac = getOptionsCache;
	ac.cache = ac.cache || {};

	if (!ac.cache[getId(id)]) {
		ac.cache[getId(id)] = parseTestOptions(getNode(id, 'test'), getQuestionsCache(id));
	}
	return ac.cache[getId(id)];
}

// Загружает оценки по тесту, сохранённые в DS
function loadSavedScores () {
	var testDataRaw = DS.get('!te') || '';

	var tests = testDataRaw.split(',');
	if (tests.length == 1 && !tests[0]) {
		return;
	}

	for (var i = tests.length - 1; i >= 0; --i) {
		var tId = tests[i].split('|')[0];
		if (!getNode(tId, 'test')) {
			continue;
		}
		_savedScores.push(tId);
		getOptionsCache(tId)['scorePercent'] = parseFloat(tests[i].split('|')[1]) || 0;
	}
}

// Сохраняет некоторые данный по тестированиям в DS
function saveTestData () {
	var tempArray = _.map(_savedScores, function(item) {
		return item + '|' + getOptionsCache(item)['scorePercent'];
	});
	DS.set('!te', tempArray.join(','));
}

// Устанавливает оценку за тест
function setScore (id, score) {
	id = getId(id);
	if (_.indexOf(_savedScores, id) == -1) {
		_savedScores.push(id);
	}
	getOptionsCache(id)['scorePercent'] = score.percent;
	getOptionsCache(id)['scoreRaw']     = score.raw;
	getOptionsCache(id)['scoreMax']     = score.max;
	getOptionsCache(id)['scoreMin']     = score.min;
	saveTestData();
}

// Расчитывает финальные параметры теста
// оценку, max/min балл и т.п.
function scoreTest (ct) {
	// Количество отвеченных вопросов
	var questionAnswered = 0;
	// Общее количество вопросов
	var totalQuestions = 0;
	// максимально возможный балл за тест
	var testScoreMax = 0;
	// минимально возможный балл за тест
	var testScoreMin = 0;
	// набранный балл за тест
	var testScore = 0;

	_.each(ct.queue.get('all'), function (cq) {
		var QScore = cq['score'];
		if (!udef(QScore)) {
			questionAnswered++;
			testScore += QScore;
		}
		totalQuestions++;
		testScoreMax += cq['scoreMax'];
		testScoreMin += cq['scoreMin'];
	});

	var percent = (testScoreMax - testScoreMin == 0)
		? 0
		: Math.round((testScore - testScoreMin) * 100 / (testScoreMax - testScoreMin));

	return {
		percent: percent,
		raw: Math.floor(testScore * 100) / 100,
		max: Math.floor(testScoreMax * 100) / 100,
		min: Math.floor(testScoreMin * 100) / 100,
		q: {
			total:    totalQuestions,
			answered: questionAnswered
		}
	};
}

// Определяет тип вопроса
function getQuestionType (Question, QuestionId) {
	// Нода <answers>
	var Answers = Question.getElementsByTagName('answers')[0];

	// Тип вопроса (multiple, single, ...)
	var QuestionType = _Xml.name(Question) == 'question'
		? attr(Question, 'type') || (Answers ? attr(Answers, 'type') : false) || 'single'
		: _Xml.name(Question).replace(/question-/, '');

	// На всякий случай
	if (Answers) {
		attr(Answers, 'type', QuestionType);
	}

	return QuestionType;
}

// Опеределяет тему вопроса
function getQuestionGroupName (Question) {
	// Тема вопроса
	// Более сложный алгоритм для eAuthor
	// проверяем родителя: если родительская нода - question-group
	// то берём атрибут name у этой ноды (Case Sensitive)
	// и количество вопросов в выборке по этой теме
	// если существует несколько тем с одинаковым именем
	// то количество вопросов будет считано из первой встретившейся
	// Если вопрос явлеятся потомком тэга test, то QuestionGroupName
	// становится '' и QuestionGroupCount устанавливается в -1
	// если по окончании цикла атрибут count всё-ещё -1,
	// то устанавливаем его значение в общее количество вопросов
	// без темы
	var QuestionGroupNode = _Xml.name(Question.parentNode) == 'question-group'
		? Question.parentNode
		: null;
	
	return QuestionGroupNode
		? norm(attr(QuestionGroupNode, 'name') || '', true)
		: '';
}

// Обнавляет хэш тем вопросов
function processQuestionGroupsHash (Question, QuestionGroups) {
	var QuestionGroupNode = _Xml.name(Question.parentNode) == 'question-group'
		? Question.parentNode
		: null;
	var QuestionGroupName = getQuestionGroupName(Question);

	// список Id вопросов по каждой теме каждый элемент хэша
	// имеет также указание о количестве вопросов, которое
	// необходимо выбрать из текущей темы, если test.selectionMode == SELECTION_MODE_FIXED
	var QuestionGroupCount = QuestionGroupNode
		? pi((attr(QuestionGroupNode, 'selection-limit') || '0'))
		: -1;
	if (QuestionGroupNode) {
		QuestionGroupCount = QuestionGroupCount > 0 ? QuestionGroupCount : 0;
	}

	if (udef(QuestionGroups[QuestionGroupName])) {
		QuestionGroups[QuestionGroupName] = {
			count: QuestionGroupCount,
			questions: []
		};
	}

	if (QuestionGroupNode && QuestionGroupName == '' && QuestionGroups['']['count'] == -1) {
		// Значит нашлась группа вопросов с пустой темой и родителем question-group
		QuestionGroups['']['count'] = QuestionGroupCount;
	}

	QuestionGroups[QuestionGroupName]['questions'].push(xid(Question));
	
	return QuestionGroups;
}

// Если вопрос оценивается по весам, то необходимо скорректировать
// максимальный и минимальный балл
function correctScoreMinMax (QuestionAnswers, QuestionType) {
	var Min = 0;
	var Max = 0;
	
	for (var l = 0, lLength = QuestionAnswers.length; l < lLength; ++l) {
		var weight = pi(attr(QuestionAnswers[l], 'weight')) || 0;
		
		if ("single" == QuestionType) {
			if (weight < Min) {
				Min = weight;
			} else if (weight > Max) {
				Max = weight;
			}
		} else {
			if (weight < 0) {
				Min += weight;
			} else if (weight > 0) {
				Max += weight;
			}
		}
	}
	
	return {
		min: Min,
		max: Max
	};
}

function getAnswerWeights (QuestionAnswers) {
	return _.reduce(QuestionAnswers, function (memo, Answer) {
		memo[xid(Answer)] = pi(attr(Answer, 'weight')) || 0;
		attr(Answer, 'weight', String(memo[xid(Answer)]));
		return memo;
	}, {});
}

// Получение классифицирующего термина в вопросе на классификацию
function getClassTerm (Answer) {
	return norm(
		(_.isElement(Answer) ? attr(Answer, 'class-group') : Answer) || '',
		true
	);
}

// Обрабатывает вопрос на классификацию
// Формирует ноду из которой XSLT преобразование
// сможет получить список классифицирующих терминов
function preprocessClassifyQuestion (Question, AnswerNodes) {
	var classTermsHash = {};
	_.each(AnswerNodes, function(Answer) {
		var ClassTerm = getClassTerm(Answer),
		    ClassTermLC = ClassTerm.toLowerCase();
		if (udef(classTermsHash[ClassTermLC])) {
			classTermsHash[ClassTermLC] = xid(Answer);
		}
		attr(Answer, 'class-group', ClassTerm);
		attr(Answer, 'class-group-lc', ClassTermLC);
		attr(Answer, 'class-group-id', classTermsHash[ClassTermLC]);
	});
}

// Обработка вопроса типа "Упражнение"
// модифицирует атрибуты тэга <object>
function preprocessExerciseQuestion (Question) {
	var O = _Xml.select(Question, {
		selector: 'object',
		single: true
	});
	if (!O) {
		return;
	}
	
	var filename = attr(O, 'filename');
	var SplittedSrc = attr(O, 'base-src').split('/');
	
	if (attr(O, 'temp-preprocessed')) {
		return O;
	}
	attr(O, 'temp-preprocessed', 'temp-preprocessed');
	attr(O, 'default', 'true');
	attr(O, 'visible', 'false');
	attr(O, 'show-in-popup', 'false');
	attr(O, 'type', 'flash');
	var exerciseFileName = /^index.htm$/.test(filename) ? "FlashExercise.swf" : (/\.htm(l)?$/i.test(filename) ? filename.split('.')[0] + ".swf" : filename);
	attr(O, 'src', SplittedSrc.join('/') + '/' + exerciseFileName);
	
	return O;
}

function getFillValidAnswers (right) {
	var isOldFormat = !(right.charAt(0) == '[' && right.charAt(right.length-1) == ']');
	// Похоже, что это новый JSON формат задания ответа
	// при новом формате появилась возможность считать любой ответ верным
	// если массив пустой, при старом формате всегда будет хотя-бы один
	// элемент в массиве!
	if (!isOldFormat)
		try { right = JSON.parse(right); } catch (error) {}
	_.isArray(right) || (right = right.split('|'));

	right = _(right).chain()
		.map(function (value) {
			return norm(value, true);
		}).uniq().value();

	if (isOldFormat && right.length > 1)
		right = _(right).without('');
	return right;
}

// Расчёт верного ответа на вопрос, если он оценивается НЕ по весам
QRightAnswer = {
	compare: function (Question, Answers, QuestionHash) {
		return QRightAnswer.fillingaps.apply(QRightAnswer, arguments);
	},
	fill: function (Question, Answers, QuestionHash) {
		return QRightAnswer.fillingaps.apply(QRightAnswer, arguments);
	},
	fillingaps: function (Question, Answers, QuestionHash) {
		return _(Answers).reduce(function (memo, ans) {
			memo[xid(ans)] = getFillValidAnswers(attr(ans, 'right') || '');
			return memo;
		}, {});
	},
	// Учитывает только первый верный ответ
	single: function (Question, Answers, QuestionHash) {
		var RightAnswers = {};
		var HasRightAnswer = false;
		_.each(Answers, function (Answer) {
			var RightAnswer = false;
			if (!HasRightAnswer) {
				RightAnswer = attr(Answer, 'type') == 'true';
				if (RightAnswer)
					HasRightAnswer = true;
			}
			RightAnswers[xid(Answer)] = RightAnswer;
		});
		return RightAnswers;
	},
	multiple: function (Question, Answers, QuestionHash) {
		var RightAnswers = {};
		_.each(Answers, function (Answer) {
			RightAnswers[xid(Answer)] = (attr(Answer, 'type') == 'true');
		});
		return RightAnswers;
	},
	sort: function (Question, Answers, QuestionHash) {
		// Хэш индексов от 1 до N
		var RightAnswers = {};
		_.each(Answers, function (Answer, index) {
			RightAnswers[xid(Answer)] = (index + 1);
		});
		return RightAnswers;
	},
	classify: function (Question, Answers, QuestionHash) {
		var RightAnswers = {};
		_.each(Answers, function (Answer) {
			//RightAnswers[xid(Answer)] = getClassTerm(Answer).toLowerCase();
			RightAnswers[xid(Answer)] = getClassTerm(Answer);
		});
		return RightAnswers;
	},
	exercise: function (Question, Answers, QuestionHash) {
		return {};
	}
}

// Обработка атрибутов ВСЕХ вопросов внутри тега <test>
function generateQuestionsHash(questions) {
	var QuestionsHash = { };
	var QuestionGroups = { };
	var RequiredQuestionsId = [];
	var AllQuestionIds = [];

	for (var i = 0, length = questions.length; i < length; ++i) {
		var Question = questions[i];

		// уникальный в пределах курса Id вопроса
		var QuestionId = xid(Question);

		// Тип вопроса (multiple, single, ...)
		var QuestionType = getQuestionType(Question, QuestionId);
		attr(Question, 'type', String(QuestionType));

		var QuestionGroupName = getQuestionGroupName(Question);
		QuestionGroups = processQuestionGroupsHash(Question, QuestionGroups);
		attr(Question, 'group-name', QuestionGroupName);

		// Если вопрос является обязательным, то заносим
		// его в список обязательных вопросов
		var QuestionIsRequired = attr(Question, 'is-required') == 'true'
			? true
			: false;
		if (QuestionIsRequired) {
			RequiredQuestionsId.push(QuestionId);
		}
		attr(Question, 'is-required', String(QuestionIsRequired));

		QuestionsHash[QuestionId] = {
			id: QuestionId,                         // id вопроса
			type: QuestionType,                     // тип вопроса
			group: QuestionGroupName,               // тема вопроса
			xmlNode: Question,                      // Xml нода вопроса
			isRequired: QuestionIsRequired,         // вопрос является обязательным?
			score: root.undefined,                  // оценка за последнюю попытку ответа на вопрос
			lastTry: root.undefined,                // описание последней попытки ответа на вопрос
			scoreDetails: root.undefined,           // замечания по последней попытке ответа на вопрос
			htmlSerializedForm: root.undefined,     // сериализованное представление последнего ответа пользователя
			tryesCount: 0,                          // количество попыток ответа на вопрос
			userAnswers: [],                        // пользовательские ответы на вопросы
			previousTryesCount: [],                 // Количество попыток ответов на вопросы в предыдущих прохождениях теста
			weights: _emptyValue,                   // Веса ответов
			scoreMin: _emptyValue,                  // минимально возможная оценка
			scoreMax: _emptyValue,                  // максимально возможная оценка
			questionComments: _emptyValue,          // комментарии к ответу
			answersFeedback: [],
			options: {},                            // какие-то специфичные для конкретного типа вопроса настройки
			                                        // например matchCase для fill и fillingaps
			                                        // нельзя использовать _emptyValue!!!
			timeLimit: _emptyValue,                 // ограничение времени для ответа на вопрос
			rightAnswer: _emptyValue,               // верный ответ в специфичном формате
			answerNodes: _emptyValue,               // массив xml нод с ответами
			shuffleAnswers: _emptyValue,            // перемешивать ответы на вопрос?
			scoreByWeights: _emptyValue             // режим оценки по весам или нет?
		};
		AllQuestionIds.push(QuestionId);
	}
	
	if (QuestionGroups[''] && QuestionGroups['']['count'] == -1) {
		QuestionGroups['']['count'] = QuestionGroups['']['questions'].length;
	}

	return {
		hash:     QuestionsHash,
		groups:   QuestionGroups,
		required: RequiredQuestionsId,
		ids:      AllQuestionIds
	};
}

function filterFeedbackNodes (ua, q, scoreDetail, feedbackNodes, id) {
	// matchCase устанавливается только для вопросов типа fill и fillingaps
	var matchCase = udef(q.options.matchCase) || q.options.matchCase;
	return _.reduce(feedbackNodes, function (memo, node) {
		var showOn = attr(node, 'show-on')
		  , showOnValue = getFillValidAnswers(attr(node, 'show-on-value') || '[]')
		  , composition = attr(node, 'composition');
		if (_.indexOf(['valid', 'invalid', 'any'], showOn) == -1)
			showOn = 'invalid';
		if (_.indexOf(['or', 'and'], composition) == -1)
			composition = 'or';
		if (!(scoreDetail === true && showOn == 'valid') && !(scoreDetail !== true && showOn == 'invalid') && showOn != 'any')
			return memo;
		if (/^(multiple|single|sort|excersise)$/.test(q['type']) || !showOnValue.length) {
			memo.push(_Xml.text(node));
		} else {
			// отфильтровать на основании ответа пользователя
			_.isArray(ua) || (ua = [ ua ]);
			ua = _(ua).reduce(function (memo, a) {
				if (!udef(a) && a != null)
					memo.push('' + a);
				return memo;
			}, []);
			matchCase || (showOnValue = _(showOnValue).invoke('toLowerCase'));
			matchCase || (ua = _(ua).invoke('toLowerCase'));

			if (composition == 'and' && (/^(compare)$/.test(q['type']) || (q.options.dd && q.options.dd[id])) && q.options.multiple && q.options.multiple[id]) {
				// если размеры массивов равны и их пересечение равно длине обоих
				if ( showOnValue.length == ua.length &&  _(showOnValue).intersect(ua).length == ua.length ) {
					memo.push(_Xml.text(node));
				}
			} else {
				if (_(showOnValue).intersect(ua).length > 0)
					memo.push(_Xml.text(node));
			}
		}
		return memo;
	}, []);
}

function questionScoreInRange (pc, comment) {
	var textRange = (attr(comment, 'treshold') || '').replace(new RegExp("(\\s|\\u00A0)", "g"), '')
	  , matches = textRange.match(/^(\[|\()(\d{1,3}),(\d{1,3})(\)|\])$/)
	  , x = [];
	if (!matches || matches.length != 5) return /^feedback$/.test(_Xml.name(comment)) || pc < 100;
	matches.shift();
	matches[1] = parseInt(matches[1], 10);
	matches[2] = parseInt(matches[2], 10);
	
	if (_(matches).first() == '[') {
		x.push(pc >= matches[1]);
	} else {
		x.push(pc > matches[1]);
	}
	if (_(matches).last() == ']') {
		x.push(pc <= matches[2]);
	} else {
		x.push(pc < matches[2]);
	}
	return x[0] && x[1];
}

QuestionClassMethods = {
	timer: function (options) {
		if (udef(options)) {
			return;
		}
		if (options === 'interrupt' && this.__timer) {
			return this.__timer.interrupt();
		} else if (_.isFunction(options.finish) && this['timeLimit']) {
			this.__timer = new eAu.Timer(this['timeLimit'], options);
		}
	},
	answer: function (userAnswer) {
		return scoreQuestion(this, userAnswer);
	},
	feedback: function (userAnswer) {
		var score = getComputedScore(this, userAnswer)
		  , q = this
		  , feedback = {}
		  , ua;
		_.each(score.scoreDetails, function (item, id) {
			if (/^(fillingaps|fill|compare)$/.test(q['type'])) {
				feedback[id] = _.map(userAnswer[id] || [], function (ua, idx) {
					return filterFeedbackNodes(ua, q, item[idx], q.answersFeedback[id] || [], id);
				});
			} else {
				feedback[id] = filterFeedbackNodes(userAnswer[id], q, item, q.answersFeedback[id] || [], id);
			}
		});
		return feedback;
	},
	comments: function (userAnswer) {
		var score = getComputedScore(this, userAnswer)
		  , q = this
		  , comments;
		comments = _(this.questionComments).reduce(function (memo, comment) {
			if (questionScoreInRange(score.scorePercent, comment)) {
				memo.push(comment);
			}
			return memo;
		}, []);
		return _(comments).map(function (comment) {
			return _Xml.text(comment);
		});
	},
	// Сбрасывает текущее состояние вопросов
	// Данные не потеряются, т.к все параметры
	// можно восстановить из других переменных
	reset: function () {
		this.previousTryesCount.push(this.tryesCount);
		this.tryesCount = 0;
		this.lastTry = root.undefined;
		this.htmlSerializedForm = root.undefined;
		this.score = root.undefined;
	}
};

// При первичном формировании списка вопросо в тэге <test>
// Многие параметры не расчитываются, что-бы не тратить время впустую
// все эти параметры расчитываются только для тех вопросов, которые
// попали в тест
function postprocessQuestionsHash (QHash, QIds, testQuestionIds) {
	_.each(QIds, function (QId) {
		if (QHash[QId]['scoreMin'] !== _emptyValue) { return; }
		var Question = QHash[QId]['xmlNode'];
		var QuestionType = QHash[QId]['type'];
		
		// Нода <answers>
		var Answers = Question.getElementsByTagName('answers')[0];
		var AnswerNodes = Answers ? _Xml.collect(Answers, 'answer') : [];
		
		// оценки по весам, но только для вопросов с типом "одиночный" и "множественный" выбор
		var QuestionScoreByWeights = /^(single|multiple)$/.test(QuestionType) && attr(Question, 'score-by-weights') == 'true'
			? true
			: false;
		attr(Question, 'score-by-weights', String(QuestionScoreByWeights));
		
		// Перемешивать ответы в вопросах (актуально не для всех типов вопросов)
		var QuestionShuffleAnswers = /sort|classify/.test(QuestionType)
			? true
			: attr(Question, 'shuffle-answers') == 'true' ? true : false;
		attr(Question, 'shuffle-answers', String(QuestionShuffleAnswers));
		
		// Минимальная и максимальная оценка за вопрос
		var QuestionScoreMin = pi(attr(Question, 'score-min') || '0');
		var QuestionScoreMax = pi(attr(Question, 'score-max') || '1');
		var QuestionWeights = {};
		if (QuestionScoreByWeights) {
			var _tmp = correctScoreMinMax(AnswerNodes, QuestionType);
			QuestionScoreMin = _tmp.min;
			QuestionScoreMax = _tmp.max;
			QuestionWeights = getAnswerWeights(AnswerNodes);
		}
		// Нормализуем значения scoreMin и scoreMax
		if (QuestionScoreMin > QuestionScoreMax) {
			QuestionScoreMin = QuestionScoreMax;
		}
		attr(Question, 'score-min', String(QuestionScoreMin));
		attr(Question, 'score-max', String(QuestionScoreMax));
		
		// Ограничение времени ответа на текущий вопрос
		var QuestionTimeLimit = (pi(attr(Question, 'time-limit')) && pi(attr(Question, 'time-limit')) >= 0)
			? pi(attr(Question, 'time-limit'))
			: 0;
		attr(Question, 'time-limit', QuestionTimeLimit);
		
		if (/^classify$/.test(QuestionType)) {
			preprocessClassifyQuestion(Question, AnswerNodes);
		} else if (/^exercise$/.test(QuestionType)) {
			AnswerNodes = [ preprocessExerciseQuestion(Question, AnswerNodes) ];
		}
		
		var QuestionComments = Answers ? _Xml.collect(Answers, 'text') : [];
		QuestionComments = QuestionComments.concat(_Xml.collect(Question, 'comments feedback'));
		QuestionComments = _.filter(QuestionComments, function (comment) {
			return !/^\s*$/.test(_Xml.text(comment) || '')
		});
		
		var QuestionRightAnswer = null;
		if (!QuestionScoreByWeights && _.isFunction(QRightAnswer[QuestionType])) {
			QuestionRightAnswer = QRightAnswer[QuestionType](Question, AnswerNodes, QHash[QId]);
		}
		
		// Правим заголовок
		if (_.indexOf(testQuestionIds, QId) != -1) {
			var qTitle = trim(attr(Question, 'title') || '');
			var qTitleRegExp = /^(Вопрос|Question|Упражнение|Exercise)(\s*(N|#|№)?\s*[0-9]*)?$/i;
			var qTitleText = qTitle.match(qTitleRegExp) ? qTitle.match(qTitleRegExp)[1] : 'Вопрос';
			var isLatin = /[A-Z]/i.test(qTitleText.charAt(0));
			if (qTitleRegExp.test(qTitle)) {
				attr(Question, 'title', qTitleText + ' ' + (isLatin ? '#' : '№') + (_.indexOf(testQuestionIds, QId) + 1));
			}
		}
		
		// Для удобства работы в XSLT
		attr(Question, 'eAu-is-question', 'yes');
		
		var feedback = collectFeedback(Question, Answers, AnswerNodes);
		
		_.extend(QHash[QId], {
			weights:  QuestionWeights,
			scoreMin: QuestionScoreMin,
			scoreMax: QuestionScoreMax,
			questionComments: QuestionComments,
			timeLimit: QuestionTimeLimit,
			answerNodes: AnswerNodes,
			answersFeedback: feedback,
			rightAnswer: QuestionRightAnswer,
			shuffleAnswers: QuestionShuffleAnswers,
			scoreByWeights: QuestionScoreByWeights
		}, QuestionClassMethods || {});
		
		// TODO дать возможность сделать postprocess для базируясь на типе вопроса
		//      как сделано с верными ответами и оцениванием.
		//      предоставить в пользование объект options
		if (/^(fill|fillingaps)$/.test(QuestionType)) {
			_.extend(QHash[QId].options, {
				matchCase: attr(Question, 'match-case') === 't' ? true : false
			});
		}
		if (/^(classify)$/.test(QuestionType)) {
			_.extend(QHash[QId].options, {
				matchCase: false
			});
		}
		if (/^(compare)$/.test(QuestionType)) {
			_.extend(QHash[QId].options, {
				matchCase: true
			});
		}
		if (/^(fillingaps)$/.test(QuestionType)) {
			_.extend(QHash[QId].options, {
				dd: _(QHash[QId]['answerNodes']).reduce(function (memo, a) {
					return (memo[xid(a)] = attr(a, 'dd') === 't', memo);
				}, {})
			});
		}
		if (/^(compare|fillingaps)$/.test(QuestionType)) {
			_.extend(QHash[QId].options, {
				multiple: _(QHash[QId]['answerNodes']).reduce(function (memo, a) {
					return (memo[xid(a)] = attr(a, 'multiple') === 't', memo);
				}, {}),
				stubs: getFillValidAnswers(attr(Question, 'stubs') || '[]'),
				perAnswerStubs: _(QHash[QId]['answerNodes']).reduce(function (memo, a) {
					if (/^(compare)$/.test(QuestionType) || QHash[QId].options.dd[xid(a)]) {
						memo[xid(a)] = getFillValidAnswers(attr(a, 'stubs') || '[]');
					}
					return memo;
				}, {})
			});
		}
	});
	
	return QHash;
}

function collectFeedback (QNode, Answers, AnswerNodes) {
	var questionFeedbackNodes = _Xml.collect(QNode, 'feedback')
	  , answersAndFeedbackNodes
	  , currentAnswerNodeId
	  , answersFeedback = {};
	if (Answers && AnswerNodes.length) {
		answersAndFeedbackNodes = _Xml.collect(Answers, 'answer feedback');
		currentAnswerNodeId = _Xml.id(AnswerNodes[0]);
		_.each(answersAndFeedbackNodes, function (node) {
			_Xml.name(node) == 'answer' && (currentAnswerNodeId = _Xml.id(node));
			if (_Xml.name(node) == 'feedback') {
				answersFeedback[currentAnswerNodeId] || (answersFeedback[currentAnswerNodeId] = []);
				if (!/^\s*$/.test(_Xml.text(node) || ''))
					answersFeedback[currentAnswerNodeId].push(node);
			}
		});
	}
	return answersFeedback;
}

// Оценка от 0 до 1
QScorers = {
	// TODO scoring can be done with fillingaps function
	compare: function (Q, UA) {
		return QScorers.fillingaps.apply(QScorers, arguments);
	},
	fill: function (Q, UA) {
		return QScorers.fillingaps.apply(QScorers, arguments);
	},
	fillingaps: function (Q, UA) {
		var details = {}
		  , answeredMax = 0  // кол-во ответов которые пользователю была дана возможность заполнить
		                     // может быть меньше количества ответов, если преподаватель забыл
		                     // проставить InnerLink, или больше, если поставил больше ссылок
		  , answeredK = 0    // кол-во ответов которые пользователь заполнил верно
		  , matchCase = Q.options.matchCase;

		_.each(Q['rightAnswer'], function (rightAnswer, index) {
			var ua = UA[index]
			  , ra = rightAnswer
			  , multiple = (udef(Q.options.dd) || Q.options.dd[index]) && !udef(Q.options.multiple) && Q.options.multiple[index];
			if (udef(ua)) return;

			matchCase || (ra = _(ra).invoke('toLowerCase'));
			matchCase || (ra = _(ra).uniq());
			ua = _(ua).map(function (ua) {
				ua = _(ua).map(function (ua) {
					ua = norm(ua, true);
					matchCase || (ua = ua.toLowerCase());
					return ua;
				});
				return _(ua).uniq();
			});

			details[index] = _(ua).map(function (ua) {
				var detail = multiple
					? (ra.length == 0 || (ua.length == ra.length && _.intersect(ua, ra).length == ra.length)) ? true : rightAnswer
					: (ra.length == 0 || _.intersect(ua, ra).length != 0) ? true : rightAnswer;

				answeredK += Number(detail === true);
				return detail;
			});
			answeredMax += ua.length;
		});
		return {
			details: details,
			score: answeredMax ? answeredK / answeredMax : 0
		};
	},
	single: function (QuestionHash, UserAnswer) {
		var details          = {},
		    UHasWrongAnswer  = false,
		    result;
		if (QuestionHash['scoreByWeights']) {
			_.each(UserAnswer, function (value, index) {
				details[index] = false;
				if (value === true && udef(result)) {
					result = QuestionHash['weights'][index];
					details[index] = true;
				}
			});
			return {
				score: udef(result) ? 0 : result,
				details: details
			};
		}
		
		// В QuestionHash['rightAnswer'] только один верный ответ!
		// Поэтому если пользователь "ухитрился" отметить два варианта ответа
		// то это будет ошибкой!!!
		// Если UserAnswer[index] будет не булевым значением, то ответ будет считаться НЕВЕРНЫМ
		_.each(QuestionHash['rightAnswer'], function (rightAnswer, index) {
			details[index] = (rightAnswer === UserAnswer[index]) ? true : String(rightAnswer);
			if (details[index] !== true)
				UHasWrongAnswer = true;
		});
		
		return {
			details: details,
			score: (UHasWrongAnswer === false) ? 1 : 0
		};
	},
	multiple: function (QuestionHash, UserAnswer) {
		var details = {}
		  , AC = { r: { c: 0, uc: 0 }, w: { c: 0, uc: 0 } };

		if (QuestionHash['scoreByWeights']) {
			return {
				score: _.reduce(UserAnswer, function (totalWeight, value, index) {
					return totalWeight + (value === true ? QuestionHash['weights'][index] : 0);
				}, 0),
				details: _.extend({}, UserAnswer)
			};
		}

		_.each(QuestionHash['rightAnswer'], function (rightAnswer, index) {
			details[index] = (rightAnswer === UserAnswer[index]) ? true : String(rightAnswer);
			details[index] === true
				? (rightAnswer ? ++AC.r.c  : ++AC.w.uc)
				: (rightAnswer ? ++AC.r.uc : ++AC.w.c)
		});
		return {
			details: details,
			score: !QuestionHash['answerNodes'].length
				? 0
				: (parseFloat(AC.r.c + AC.w.uc - AC.r.uc - AC.w.c)
					/ parseFloat(QuestionHash['answerNodes'].length) / 2.0 + 0.5)
		};
	},
	sort: function (QuestionHash, UserAnswer) {
		var RightAnswersCount = 0
		  , details = {}
		  , UAIsArray = _.isArray(UserAnswer);
		_.each(QuestionHash['rightAnswer'], function (rightAnswer, index) {
			var userAnswerIndex;
			if (UAIsArray) {
				userAnswerIndex = _.indexOf(UserAnswer, index) + 1;
			} else {
				userAnswerIndex = UserAnswer[index];
			}
			details[index] = rightAnswer === userAnswerIndex ? true : rightAnswer;
			RightAnswersCount += Number(details[index] === true);
		});
		return {
			details: details,
			score: !QuestionHash['answerNodes'].length
				? 0
				: parseFloat(RightAnswersCount) / QuestionHash['answerNodes'].length
		};
	},
	classify: function (QuestionHash, UserAnswer) {
		var RightAnswersCount = 0,
		    details = {};
		_.each(QuestionHash['rightAnswer'], function (rightAnswer, index) {
			var ua = str(UserAnswer[index]) ? getClassTerm(UserAnswer[index]).toLowerCase() : _emptyValue
			  , ra = rightAnswer.toLowerCase();
			details[index] = ua === ra ? true : rightAnswer;
			RightAnswersCount += Number(details[index] === true);
		});
		return {
			details: details,
			score: !QuestionHash['answerNodes'].length
				? 0
				: parseFloat(RightAnswersCount) / parseFloat(QuestionHash['answerNodes'].length)
		};
	},
	exercise: function (QuestionHash, UserAnswer) {
		// TODO: что делать когда UserAnswer пуст?
		//       сейчас я буду извещать пользователя об ошибке и ставить минимальную оценку
		if (udef(UserAnswer['comletion_status']) || udef(UserAnswer['score.raw'])) {
			return {
				details: 'Invalid',
				score: 0
			}
		} else {
			return {
				score: UserAnswer['comletion_status'] == 'completed' && (UserAnswer['score.max'] - UserAnswer['score.min'])
					? ((UserAnswer['score.raw'] - UserAnswer['score.min']) / (UserAnswer['score.max'] - UserAnswer['score.min']))
					: 0
			};
		}
	}
};

function scoreQuestion (QuestionHash, UserAnswer) {
	QuestionHash['tryesCount'] += 1;
	
	if (!UserAnswer) {
		throw new Error("Unable to score question, there is no user answer supplied");
	}
	
	QuestionHash['userAnswers'].push(UserAnswer);
	QuestionHash['lastTry'] = UserAnswer;
	
	if (!_.isFunction(QScorers[ QuestionHash['type'] ])) {
		throw new Error("Unable to score question, there is no scoring function for question type '"+ QuestionHash['type'] +"'");
	}
	
	_.extend(QuestionHash, getComputedScore(QuestionHash, UserAnswer));
	
	return QuestionHash['score'];
}
function getComputedScore (QuestionHash, UserAnswer) {
	var result = {};

	if (!UserAnswer)
		throw new Error("Unable to score question, there is no user answer supplied");
	if (!_.isFunction(QScorers[ QuestionHash['type'] ]))
		throw new Error("Unable to score question, there is no scoring function for question type '"+ QuestionHash['type'] +"'");

	result = QScorers[ QuestionHash['type'] ](QuestionHash, UserAnswer);

	if (!QuestionHash['scoreByWeights'])
		result.score = QuestionHash['scoreMin'] + (QuestionHash['scoreMax'] - QuestionHash['scoreMin']) * result.score;

	result.scoreDetails = result.details;
	if (QuestionHash['scoreMax'] == QuestionHash['scoreMin'])
		result.scorePercent = 100;
	else
		result.scorePercent = (result.score - QuestionHash['scoreMin']) * 100 / (QuestionHash['scoreMax'] - QuestionHash['scoreMin']);

	return (delete result.details, result);
}

// На основе атрибутов тэга <question*> генерируется выборка
// вопросов для теста
function generateTest (TestOptions, params) {
	var TestQuestions = [];
	var rand = TestOptions['shuffleQuestions'];
	var HitTestHash = {};

	var QuestionGroups = _.reduce(params.groups, function (memo, value, key) {
		memo[key] = _.clone(value);
		return memo;
	}, { });

	// Сначала выбираем все обязательные вопросы, если необходимо перемешать,
	// то перемешаем их до начала выборки
	var RequiredQuestionIds = rand
		? _.clone(params.required)
		: eAu.Array.shuffle(params.required, false, true);

	while (RequiredQuestionIds.length) {
		// Добавляем Id вопроса в список вопросов теста
		var QId = RequiredQuestionIds.shift();
		TestQuestions.push(QId);

		// помечаем, что вопрос с данным Id уже присутствует в тесте
		// и на следующем этапе выборки не следует их брать
		HitTestHash[QId] = true;

		// FIXME: нужно ли уменьшать счётчик выборки из группы??
		QuestionGroups[ params.hash[QId]['group'] ]['count'] -= 1;
	}

	switch(TestOptions['selectionMode']) {
		case SELECTION_MODE_EQUAL:
			// Отнимаем то количество вопросов, которое уже добавлено в тест
			var nOfQuestions = TestOptions['numberOfQuestions'] - TestQuestions.length;

			// Теперь берём из каждой группы по вопросу пока не исчерпаем лимит
			// если необходимо перемешивание, то перемешаем группы
			var Groups = _.keys(QuestionGroups);
			if (rand) { Groups = eAu.Array.shuffle(Groups); }

			while (nOfQuestions > 0) {
				var LeftQuestions = 0;
				for (var j = 0; j < Groups.length; ++j) {
					if (nOfQuestions <= 0) { break; }

					// Получаем список вопросов в текщей теме
					var questions = _.clone(QuestionGroups[Groups[j]]['questions']);

					// Перемешиваем их, если потребуется
					if (rand) { questions = eAu.Array.shuffle(questions); }

					// до тех пор пока не найдём "свежего" вопроса или
					// пока вопросы не кончатся, ищем вопрос
					while (questions.length) {
						var QId = questions.shift();
						if (HitTestHash[QId]) { continue; }

						// если нашли подходящий вопрос, то добавляем его в список
						TestQuestions.push(QId);
						HitTestHash[QId] = true;
						--nOfQuestions;
						break;
					}
					LeftQuestions += questions.length;
				}

				if (LeftQuestions == 0) { break; }
			}
			break;
		case SELECTION_MODE_FIXED:
			// Теперь берём из каждой группы по необходимому количеству вопросов
			// здесь можно обойтись и без перемешивания
			var Groups = _.keys(QuestionGroups);
			_.each(QuestionGroups, function (value, key) {
				// Получаем список вопросов в текщей теме
				var questions = _.clone(value['questions']);
				// Перемешиваем их, если потребуется
				if (rand) { questions = eAu.Array.shuffle(questions); }

				// узнаём количество вопросов, которое необходимо выбрать из темы
				var questionsN = value['count'];

				// пока есть вопросы в теме добавляем их в тест
				for (var i = 0, length = questions.length; i < length; ++i) {
					// если достигли ограничения - прерываемся
					if (questionsN <= 0) { break; }

					var QId = questions[i];
					if (HitTestHash[QId]) { continue; }
					// нашли подходящий вопрос, добавляем
					TestQuestions.push(QId);
					HitTestHash[QId] = true;
					--questionsN;
				}
			});
			break;
	}

	if (rand) { TestQuestions = eAu.Array.shuffle(TestQuestions); }

	return TestQuestions;
}

// Получение вопроса из очереди вопросов
QGetters = {
	next: function () {
		var startIndex = this._ipp * (this._currentPosition + 1),
		    endIndex = this._ipp * (this._currentPosition + 2),
		    QIds;
		if (this._test.getOption('navigationMode') == NAVIGATION_MODE_ALLOWSKIP
				&& this.getLeft() <= this._ipp) {
			return QGetters.current.call(this);
		}
		
		return this._testQuestionQueue.slice(startIndex, endIndex);
	},
	previous: function () {
		var startIndex = this._ipp * (this._currentPosition - 1),
		    endIndex = this._ipp * this._currentPosition;
		return this._testQuestionQueue.slice(startIndex < 0 ? 0 : startIndex, endIndex);
	},
	current: function () {
		return this._testQuestionQueue.slice(
			this._currentPosition,
			this._ipp * (this._currentPosition + 1)
		);
	},
	all: function () {
		return this._testQuestionIds;
	}
};

// Для удобства
QGetters.c = QGetters.cur = QGetters.current;
QGetters.n = QGetters.next;
QGetters.p = QGetters.prev = QGetters.previous;
QGetters.a = QGetters.all;

QuestionQueueClass = eAu.Class.create({
	initialize: function (testObject, questions) {
		this._test = testObject;
		var params = generateQuestionsHash(questions);

		this._questionsHash = params.hash;
		this._allTestQuestionIds = params.ids;
		this._testQuestionIds = generateTest(this._test._options, params);
		if (this._test._options['itemsPerPage'] == 0 || this._test._options['itemsPerPage'] > this._testQuestionIds.length) {
			this._test._options['itemsPerPage'] = this._testQuestionIds.length;
		}
		this._ipp = this._test._options['itemsPerPage'];
		// Теперь можно приступить к вычислению всех необходимых параметров
		// вопроса. Раньше это делалось в generateQuestionsHash
		this.reset();
	},
	reset: function () {
		if (this._testQuestionQueue) {
			_.each(this._testQuestionIds, _.bind(function (item) {
				this.get(item).reset();
			}, this));
		}
		this._testQuestionQueue = _.clone(this._testQuestionIds);
		this._currentPosition = 0;
	},
	get: function (what) {
		var itemIds,
		    questions;
		what = (what || 'current');

		if (_.isFunction(QGetters[what.toLowerCase()])) {
			itemIds = QGetters[what.toLowerCase()].call(this);
		} else if (!udef(this._questionsHash[what])) {
			itemIds = [what];
		}
		
		if (itemIds) {
			questions = _.map(itemIds, _.bind(function (QId) {
				return this._questionsHash[QId];
			}, this));
			this._questionsHash = postprocessQuestionsHash(this._questionsHash, itemIds, this._testQuestionIds);
			return _.isFunction(QGetters[what.toLowerCase()]) ? questions : questions[0];
		}
	},
	// Является-ли текущая страница - последней?
	// Если страница последняя, это не значит что get('next') вернёт []
	isLast: function () {
		return this.getLeft() <= this._ipp;
	},
	isEmpty: function () {
		return this._testQuestionIds.length == 0;
	},
	getPosition: function (Question) {
		var position = _.indexOf(this._testQuestionIds, Question['id']);
		return position == -1 ? -1 : position + 1;
	},
	getLength: function () {
		return this._testQuestionIds.length;
	},
	getLeft: function () {
		return this._testQuestionQueue.length - this._currentPosition;
	},
	previous: function () {
		// Если нам запрещено возвращаться назад, то прерываем функцию
		if (this._test.getOption('navigationMode') != NAVIGATION_MODE_BIDIRECTIONAL) {
			return;
		}

		// Иначе, перемещаем указатель текущей позиции назад
		// Никаких проверок, пусть ui этим занимается
		this._currentPosition -= this._ipp;
		this._currentPosition = this._currentPosition < 0 ? 0 : this._currentPosition;
		
		this._resetQuestionStatuses();
	},
	// Очищает статусы 
	// для показа текущей страницы
	current: function () {
		this._resetQuestionStatuses();
	},
	// Просто модифицируем очередь
	advance: function () {
		if (!this.pageStatusSet())
			return;
		
		_.each(this.get(), _.bind(function (Question, index) {
			var status = this.__statuses[index];
			if (status == 'answer') {
				if (this._test.getOption('navigationMode') != NAVIGATION_MODE_BIDIRECTIONAL) {
					// Если запрещён возврат к предыдущему вопросу, то не имеет смысла
					// хранить его Id в очереди, просто удаляем его
					this._testQuestionQueue.shift();
				} else {
					// Иначе, перемещаем указатель текущей позиции вперёд
					// Никаких проверок, пусть ui этим занимается
					this._currentPosition++;
				}
			} else if (status == 'skip') {
				// извлекаем вопрос с начала очереди и добавляем его в самы конец
				this._testQuestionQueue.push( this._testQuestionQueue.shift() );
			} else {
				// TODO: test & remove
				//throw new Error('QuestionQueueClass.advace: invalid status: ' + status);
			}
		}, this));
		
		this._resetQuestionStatuses();
	},
	getQuestionsWithStatus: function (status) {
		var questions = [],
		    currentQuestions = this.get();
		status = status || 'answer';
		
		_.each(this.__statuses || [], _.bind(function (currentStatus, index) {
			if (currentStatus == status)
				questions.push(currentQuestions[index]);
		}, this));
		
		return questions;
	},
	getQuestionsWithUndefinedStatus: function () {
		var undefinedStatuses = [];
		
		if (!this.__statuses)
			this._resetQuestionStatuses();
		
		_.each(this.get(), _.bind(function (Question, index) {
			if (_.isNumber(this.__statuses[index])) {
				undefinedStatuses.push(Question);
			}
		}, this));
		
		return undefinedStatuses;
	},
	answer: function (Question) {
		return this._setQuestionStatus(Question, 'answer', true);
	},
	skip: function (Question) {
		if (this._test.getOption('navigationMode') != NAVIGATION_MODE_ALLOWSKIP) {
			return;
		}
		return this._setQuestionStatus(Question, 'skip', true);
	},
	getStatus: function (Question) {
		var index = _.indexOf(this.get(), Question);
		if (index == -1 || !this.__statuses || _.isNumber(this.__statuses[index]))
			return '';
		
		return this.__statuses[index];
	},
	_setQuestionStatus: function (Question, status, force) {
		var index = _.indexOf(this.get(), Question);
		if (index == -1)
			return;
		
		if (!this.__statuses)
			this._resetQuestionStatuses();
		
		// TODO: test & remove
		if (this.__statuses.length != this.get().length)
			throw new Error('QuestionQueueClass._setQuestionStatus: status queue is of invalid length');
		
		if (!_.isNumber(this.__statuses[index]) && !force)
			return;
		
		this.__statuses[index] = status;
		return true;
	},
	_resetQuestionStatuses: function () {
		this.__statuses = _.range(this.get().length);
	},
	pageStatusSet: function () {
		return !!this.__statuses && !_.detect(this.__statuses, function (status) { return _.isNumber(status); });
	},
	isSkipAllowed: function () {
		return this._test.getOption('navigationMode') == NAVIGATION_MODE_ALLOWSKIP;
	},
	isPreviousAllowed: function () {
		return this._test.getOption('navigationMode') == NAVIGATION_MODE_BIDIRECTIONAL;
	}
});

TestClass = eAu.Class.create({
	initialize: function (test) {
		this.__testNode = test;
		this.__testId = xid(this.__testNode);
		this.__questions = getQuestionsCache(this.__testId);
		this.__timer = null;
		// Получаем настройки тестирования
		this._options = getOptionsCache(this.__testId);
	},
	start: function () {
		if (this.__started) {
			return;
		}

		if (this.queue) {
			this.queue.reset();
		} else {
			this.queue = new QuestionQueueClass(this, this.__questions);
		}

		this.__result = {
			startTime: (new Date()).getTime()
		};

		this.__finished = false;
		this.__started = true;
	},
	finish: function (finishMode) {
		if (!this.__started || this.__finished) {
			return;
		}

		this.__result['endTime'] = (new Date()).getTime();

		this.__finished = true;
		this.__started = false;
	},
	isFinished: function () {
		return !!this.__finished;
	},
	score: function (saveScore) {
		var score = scoreTest(this);
		if (saveScore) {
			setScore(this.__testId, score);
		}
		return score;
	},
	getTestNode: function () {
		return this.__testNode;
	},
	getOption: function (name) {
		return eAu.Test.getOption(this.__testId, name);
	},
	passed: function () {
		return eAu.Test.passed(this.__testId);
	},
	attempted: function () {
		return eAu.Test.attempted(this.__testId);
	},
	showRightAnswers: function () {
		return !!this.getOption('showRightAnswers');
	},
	showStat: function () {
		return !!this.getOption('showTestStats');
	},
	timer: function (options) {
		if (udef(options)) {
			return;
		}
		if (options === 'interrupt' && this.__timer) {
			return this.__timer.interrupt();
		} else if (_.isFunction(options.finish) && this.getOption('timeLimit')) {
			this.__timer = new eAu.Timer(this.getOption('timeLimit'), options);
		}
	},
	isInterruptAllowed: function () {
		return this.getOption('allowTestInterrupt');
	}
});

eAu.Test = {
	init: function (DataStorageEngine) {
		DS = DataStorageEngine;
		loadSavedScores();
		saveTestData();
	},
	get: function (testNode) {
		if (!getCachedTest(testNode)) {
			setCachedTest(new TestClass(testNode));
		}
		return getCachedTest(testNode);
	},
	create: function (testNode) {
		CurrentTest = eAu.Test.get(testNode);
		return CurrentTest;
	},
	getCurrent: function () {
		return CurrentTest;
	},
	getOption: function (id, name) {
		return getOptionsCache(id)[name];
	},
	passed: function (id) {
		var options = getOptionsCache(id);
		var scorePercent = options['scorePercent'];
		var passedPercent = options['passedPC'];

		return eAu.Test.attempted(id) && (scorePercent >= passedPercent);
	},
	attempted: function (id) {
		return !udef(getOptionsCache(id)['scorePercent']);
	},
	isQuestionNode: function (idOrNodeName) {
		return isQuestionNode(idOrNodeName);
	}
}

})(this);
