;(function() {

var root = this;

if (root.eAu.Timer) { return; }

root.eAu.Timer = eAu.Class.create({
	initialize: function(duration, options) {
		if (!_.isNumber(duration) || isNaN(duration) || duration <= 0) {
			return;
		}
		this._options = _.extend({
			start:     _.identity, /* Исполняется при старте таймера */
			finish:    _.identity, /* Исполняется по завершении таймера */
			callback:  _.identity, /* Исполняется в фиксированные промежутки времени */
			interrupt: _.identity, /* Исполняется при прерывании таймера */
			onError:   _.identity,
			precision: 250, /* Как часто будет исполняться callback */
			delay: 0 /* Пока не работает */
		}, options || {});
		var date = (new Date());
		this._startTime = this._currentTime = date.getTime();
		this._timezone = date.getTimezoneOffset();
		this._duration = this._options.duration = duration;
		this._iteration = 0;
		this._start();
	},
	_start: function() {
		if (this.isIdle()) { return; }
		
		_.defer(_.bind(function () { this._options.start(this._duration); }, this));
		this._iterate();
	},
	_iterate: function() {
		if (this.isIdle()) { return; }
		
		if (this._duration <= 0) {
			this._finish();
			return;
		}
		
		var currentTime = new Date();
		var time = currentTime.getTime();
		if ( time < this._currentTime || currentTime.getTimezoneOffset() != this._timezone ) {
			this._onError('Произошла смена часового пояса или изменение времени');
			return;
		}
		this._duration -= time - this._currentTime;
		this._currentTime = time;
		this._iteration++;
		this._callback();
		var timeout = (this._options.precision < this._duration)
			? this._options.precision
			: (this._duration > 0) ? this._duration : 0;
		_.delay(_.bind(this._iterate, this), timeout);
	},
	_onError: function(errorText) {
		this._options.onError(this._duration, errorText || '');
	},
	_callback: function() {
		_.defer(_.bind( function () {
			if (!this.isIdle()) {
				this._options.callback(this._duration > 0 ? this._duration : 0); 
			}
		}, this));
	},
	_finish: function() {
		this._iteration = eAu.undefined;
		this._options.finish();
	},
	isIdle: function() {
		return _.isUndefined(this._iteration);
	},
	interrupt: function() {
		if (!this.isIdle()) {
			this._options.interrupt(this._duration);
			this._iteration = eAu.undefined;
		}
		var timespan = (new Date()).getTime() - this._startTime;
		return (timespan <= this._options.duration) ? timespan : this._options.duration;
	}
});

})();
