<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
		xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	<xsl:output method="html" encoding="UTF-8" />

	<xsl:param name="element-title"/>

	<!-- Удаляет фигурные скобки вначале и вконце строки -->
	<xsl:template name="stripBraces">
		<xsl:param name="string"/>
		<xsl:choose>
			<xsl:when test="substring($string,1,1)='{' and substring($string,string-length($string))='}'">
				<xsl:value-of select="translate(substring($string,2,string-length($string)-2), '-', '_')"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$string"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!-- Генерирует более приемлемый id на основе DB_ID -->
	<xsl:template name="genID">
		<xsl:param name="type">ID</xsl:param>
		<xsl:param name="id" />
		<xsl:value-of select="$type" />_<xsl:call-template name="stripBraces">
			<xsl:with-param name="string" select="$id" />
		</xsl:call-template><xsl:text>_EU</xsl:text>
	</xsl:template>

	<!-- Нормализация значения атрибута @width у тега <object> -->
	<xsl:template name="getWidth">
		<xsl:param name="node"/>
		<xsl:choose>
			<xsl:when test="$node/@width and $node/@width != '0' and $node/@width != ''">
				<xsl:value-of select="$node/@width"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:choose>
					<xsl:when test="$node/@type = 'image'">
						<xsl:text>-1</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text>800</xsl:text>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<!-- Нормализация значения атрибута @height у тега <object> -->
	<xsl:template name="getHeight">
		<xsl:param name="node"/>
		<xsl:choose>
			<xsl:when test="$node/@height and $node/@height != '0' and $node/@height != ''">
				<xsl:value-of select="$node/@height"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:choose>
					<xsl:when test="$node/@type = 'image'">
						<xsl:text>-1</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text>600</xsl:text>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<xsl:template name="getAlign">
		<xsl:param name="node"/>
		<xsl:choose>
			<xsl:when test="$node/@align = 'center' or $node/@align = 'left' or $node/@align = 'right'">
				<xsl:value-of select="$node/@align"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:text>left</xsl:text>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="object">
		<xsl:if test="not(@visibility = 'hidden')">
			<div class="eAu-object">
				<xsl:if test="$element-title and not(@visibility = 'hidden')">
					<h1 class="eAu"><xsl:value-of select="$element-title"/></h1>
				</xsl:if>
				<xsl:if test="not(@visible = 'false')">
					<div class="eAu-object-link">
						<img alt="{@type}" vspace="2" hspace="2" align="absmiddle" border="0" src="images/media/{@type}.gif"/>
						<a href="index.htm" InnerLink="EUL:{@DB_ID}" target="_top"><xsl:value-of select="@title" /></a>
					</div>
				</xsl:if>
				<xsl:if test="@default = 'true'">
					<div class="eAu-object-embed">
						<xsl:variable name="width"><xsl:call-template name="getWidth"><xsl:with-param name="node" select="."/></xsl:call-template></xsl:variable>
						<xsl:variable name="height"><xsl:call-template name="getHeight"><xsl:with-param name="node" select="."/></xsl:call-template></xsl:variable>
						<xsl:variable name="align"><xsl:call-template name="getAlign"><xsl:with-param name="node" select="."/></xsl:call-template></xsl:variable>
						<xsl:variable name="id"><xsl:call-template name="genID"><xsl:with-param name="id" select="@DB_ID" /></xsl:call-template></xsl:variable>
						<xsl:choose>
							<xsl:when test="@type = 'image'">
								<img alt="{@title}" title="{@title}" src="{@src}" align="{$align}">
									<xsl:if test="$width != -1">
										<xsl:attribute name="width"><xsl:value-of select="$width"/></xsl:attribute>
									</xsl:if>
									<xsl:if test="$height != -1">
										<xsl:attribute name="height"><xsl:value-of select="$height"/></xsl:attribute>
									</xsl:if>
								</img>
								<br clear="all" />
							</xsl:when>
							<xsl:otherwise>
								<div class="eAu-object-alignment-container eAu-object-alignment-{$align}">
									<div style="width: {$width}px; height: {$height}px;" class="eAu-object-alignment">
										<xsl:choose>
											<xsl:when test="@type = 'flash'">
												<div style="background: white;"><div id="{$id}" /></div>
												<img src="images/loaded.gif">
													<xsl:attribute name="onload">
														if (!inDOM.call(this)) { return; }

														embedFlashMovie(
															'<xsl:value-of select="@DB_ID" />',
															{
																width:  '<xsl:value-of select="$width" />',
																height: '<xsl:value-of select="$height" />'
															}
														);

														this.parentNode.removeChild(this);
													</xsl:attribute>
												</img>
											</xsl:when>
											<xsl:otherwise>
												<iframe frameborder="0" marginheight="0" marginwidth="0" src="javascript:void(0);" style="display: none;" width="100%" height="100%" id="{$id}"/>
												<img src="images/loaded.gif">
													<xsl:attribute name="onload">
														if (!inDOM.call(this)) { return; }
														<xsl:choose>
															<xsl:when test="@eAu-supported-extension and @eAu-supported-extension = 'false'">
																$('#<xsl:value-of select="$id"/>')
																	.attr('src', 'not-supported-extension.htm?extension=<xsl:value-of select="@eAu-filename-extension"/>&amp;filename=' + encodeURIComponent('<xsl:value-of select="@xslt-src"/>') )
																	.show();
															</xsl:when>
															<xsl:otherwise>
																$('#<xsl:value-of select="$id"/>')
																	.attr('src', encodeURI('<xsl:value-of select="@xslt-src"/>'))
																	.show();
															</xsl:otherwise>
														</xsl:choose>
														this.parentNode.removeChild(this);
													</xsl:attribute>
												</img>
											</xsl:otherwise>
										</xsl:choose>
									</div>
								</div>
							</xsl:otherwise>
						</xsl:choose>
					</div>
				</xsl:if>
			</div>
		</xsl:if>
	</xsl:template>

	<xsl:template match="text">
		<xsl:if test="not(@visibility) = 'hidden'">
			<div class="eAu-paragraph-container">
				<xsl:if test="$element-title">
					<h1 class="eAu"><xsl:value-of select="$element-title"/></h1>
				</xsl:if>
				<div class="eAu-paragraph"><xsl:value-of select="."/></div>
			</div>
		</xsl:if>
	</xsl:template>

	<xsl:template name="studiedproblem">
		<xsl:param name="heading" select="'h1'"/>
		<xsl:param name="node"/>
		<div class="eAu-studiedproblem-container">
			<xsl:element name="{$heading}">
				<xsl:attribute name="class">eAu</xsl:attribute>
				<xsl:value-of select="$node/@title" />
			</xsl:element>
			<div class="eAu-studiedproblem"><xsl:apply-templates select="$node/*" /></div>
		</div>
	</xsl:template>

	<xsl:template name="modules">
		<xsl:param name="node" />
		<div class="eAu-modules-container">
			<h1 class="eAu"><xsl:value-of select="$node/@title" /></h1>
			<div class="eAu-modules">
				<xsl:for-each select="$node/*">
					<xsl:if test="name(.) = 'lesson'">
						<div class="eAu-appendix-lesson">
							<xsl:call-template name="modules">
								<xsl:with-param name="node" select="." />
							</xsl:call-template>
						</div>
					</xsl:if>
					<xsl:if test="name(.) = 'studiedproblem'">
						<xsl:call-template name="studiedproblem">
							<xsl:with-param name="node" select="."/>
							<xsl:with-param name="heading" select="'h2'"/>
						</xsl:call-template>
						<xsl:if test="position() != count(../*)">
							<div class="eAu-content-dotted-line"></div>
						</xsl:if>
					</xsl:if>
				</xsl:for-each>
			</div>
		</div>
	</xsl:template>

	<xsl:template match="appendix-item">
		<div class="eAu-appendix-item-container">
			<h1 class="eAu"><xsl:value-of select="@title" /></h1>
			<xsl:apply-templates select="intro[1] | modules/studiedproblem | modules/lesson"/>
		</div>
	</xsl:template>
	
	<xsl:template match="intro">
		<div class="eAu-intro-container">
			<h2 class="eAu"><xsl:value-of select="@title" /></h2>
			<div class="eAu-intro">
				<xsl:choose>
					<xsl:when test="object"><xsl:apply-templates select="object" /></xsl:when>
					<xsl:otherwise><xsl:value-of select="." /></xsl:otherwise>
				</xsl:choose>
			</div>
		</div>
	</xsl:template>

	<xsl:template match="lesson">
		<div class="eAu-appendix-lesson">
			<xsl:call-template name="modules">
				<xsl:with-param name="node" select="." />
			</xsl:call-template>
		</div>
	</xsl:template>

	<xsl:template match="studiedproblem">
		<xsl:call-template name="studiedproblem"><xsl:with-param name="node" select="."/></xsl:call-template>
	</xsl:template>
</xsl:stylesheet>
